byte data[22];
int checksum_flag;
int j,k;
char str[128];
int PWM=0; 
int repeat_n = 10000;

#define def_PWM 75 // デフォルトduty比は75とする


void setup() {
  pinMode(3,OUTPUT); // モータPWM制御ピン
  Serial.begin(115200); // PC間シリアルポートを115200 bps[ビット/秒]で初期化
  while(!Serial);
  Serial1.begin(115200);// XV-11間シリアルポートを115200 bps[ビット/秒]で初期化

  while(command_read()); // スタートコマンドが入力されるまで待つ
  analogWrite(3,PWM); // コマンドで指定されたduty比(PWM)でモータをPWM制御
}


void loop() {

while(1){

  while(xv11_dataread(data)); // 0xFAで始まる22byteのデータを読むまで待つ
  xv11_printdata(data); // 22byteのデータを画面に表示 
  checksum_flag = xv11_checksum(data); // チェックサムをチェック
//  Serial.print("checksum_flag = ");
//  Serial.println(checksum_flag);
  if(checksum_flag){
    for(j=0;j<4;j++){
      if(xv11_invalidflag(data,j)){
        sprintf(str,",%04d,0,%03d,0000,%03d,%03d",k,xv11_angle(data,j),xv11_speed(data),PWM);
        Serial.println(str);
      }
      else{
        sprintf(str,",%04d,1,%03d,%04d,%03d,%03d",k,xv11_angle(data,j),xv11_distance(data,j),xv11_speed(data),PWM);
        Serial.println(str);
      }
      k++;
      if(k>repeat_n) break; // forループから抜ける
      if(k>9999){
        k=0;
      }
    }
  }
  if(k>repeat_n) break; // whileループから抜ける
  else{
    sprintf(str,",%04d,0,000,0000,000,%03d",k,PWM);
    Serial.println(str);
    k++;
    if(k>repeat_n) break; // whileループから抜ける
    if(k>9999){
      k=0;
    }
  }
  
  if(Serial.available()){ // コマンドが入力されたら
    command_read(); // スタートコマンドが入力されるまで待つ
    analogWrite(3,PWM); // コマンドで指定されたduty比(PWM)でモータをPWM制御
  }
  
}

// リピート回数に達したら
k=0;
Serial.println("All end");
analogWrite(3,0); // モーター停止
while(command_read()); // スタートコマンドが入力されるまで待つ
analogWrite(3,PWM); // コマンドで指定されたduty比(PWM)でモータをPWM制御
}





/* スタートビット0xFAを読み込んだら22byteのデータを読み込み0を返す関数 */
int xv11_dataread(byte data[]){
  if(Serial1.available()){
    data[0]=Serial1.read();
    if(data[0]==0xFA){
      for(int i=1; i<22; i++){
        if(Serial1.available()){
          data[i]=Serial1.read();
        }
      }
      return 0;
    }
  }
  if(Serial.available()) return 0; // PCからコマンドが入力されたらやめる
  return 1; // スタートビットを読み込まなかったら1を返す
}


/* 22byteのデータが入った配列data[]を画面に表示する関数 */
void xv11_printdata(byte data[]){
  char hex_string[8];
  for(int i=0; i<22; i++){ // 22byteをPCに送信
    sprintf(hex_string,"%02x",data[i]);
    Serial1.print(hex_string);
  }
  Serial1.println();
}


/* チェックサムを計算する関数 */
int xv11_checksum(byte data[]){
  
  unsigned int data_list[10]; // 16bit
  unsigned long int chk32=0; // 32bit
  unsigned int checksum=0;
  unsigned int checksum2=0;
  int i;
  
  for(i=0;i<10;i++){
      data_list[i] = data[2*i]+(data[2*i+1]<<8); //data[0]からdata[19]まで
      }  
  for(i=0;i<10;i++){
    chk32=(chk32<<1) + data_list[i];
    }
  
  checksum=(chk32 & 0x7FFF) + (chk32>>15);
  checksum=(checksum & 0x7FFF);
  checksum2 = data[21] << 8; // checksum_H 8bitシフト
  checksum2 = checksum2 | data[20]; // checksum_L

  Serial1.println(checksum); // これがないとなぜか正常動作しない
  
  if(checksum2 == checksum){
    return 1;
    }
  return 0;
}


/* n番目のdataのinvalid flagを調べる関数 */
int xv11_invalidflag(byte data[],int n){
  if ((data[5+4*n] & 0x80) >> 7){ // invalid flagはdataの5,9,13,17番目の先頭1bit
    return 1; // invalid flagが1なら1を返す
  }
  return 0;
}


/* n番目のdataのdistanceを調べる関数 */
int xv11_distance(byte data[],int n){
  unsigned int distance;
  distance=data[5+4*n]; //距離は、上位6bitがdata_decの5,9,13,17番目の下6bit、
  distance=distance & 0x3F;
  distance=distance<<8;
  distance=distance|data[4+4*n]; //下位8itが4,8,12,16番目
  return distance;
}


/* 回転速度を調べる関数 */
int xv11_speed(byte data[]){
  //float seisu,syousu,spd;
  int seisu; // Arduinoのsprintfではfloat型を文字列に変換できないので、整数で表示する
  seisu=(data[3]<<2)|(data[2]>>6);
  //syousu=data[2]&0x3F;
  //spd=seisu + syousu*(0.015625);
  return seisu;
}


/* 角度を調べる関数 */
int xv11_angle(byte data[],int n){
  int angle;
  angle=(data[1]-0xA0)*4 + n;
  return angle;
}


/* 特定の文字swがくるまでSerial.readする関数 */
void Serial_gets(char buff[],char sw) {
  int i=0;
  char c;
  while(1) {
    if(Serial.available()) {
      c=Serial.read();
      buff[i]=c;
      if (c==sw) break;
      i++;
    }
  }
buff[i]='\0';
}



/* コマンド入力を処理する関数 */
int command_read(){
  int i=0;
  char command_str[64];
  char str;
  char command_val[64];
  int val;
  
  Serial_gets(command_str,'\r'); // 改行コード\rがくるまで入力された文字を読み込む
  str=command_str[0]; // 読み込まれた文字列の最初の1文字はコマンドの種類をあらわす文字
  // 読み込まれた文字の3文字目以降は数字をあらわす
  while(1){
    if(command_str[i+2]<'0' || '9'<command_str[i+2]){
      break;
    }
    else{
      command_val[i]=command_str[i+2];
      i++;
      if(i>3) break; // 5桁以上は受け付けない
    }
  }
  command_val[i]='\0';
  val=atoi(command_val); // 数字をあらわす文字列を数値に変換
//  Serial.println(str);
//  Serial.println(val);

  switch(str){
    
    case 's': // スタート 繰り返し回数val回
      Serial.println("start");
      k=0; // 回数カウントリセット
      if(0<val && val<10000){
        repeat_n = val; // val回データを取得する
        if(PWM==0){
          PWM=def_PWM; // duty比が0だったらdef_duty比にしないと動かない
        }
      }
      else{
        repeat_n = 10000; // データ取得回数無制限とする
        if(PWM==0){
          PWM=def_PWM; // duty比が0だったらdef_duty比にしないと動かない
        }
      }
      return 0;
      
    case 'e': // ストップ（エンド）
      PWM=0;
      Serial.println("end");
      break;
      
    case 'p': // PWM duty設定
      if(0<val && val<256){
        PWM=val;
      }
      else{
        PWM=0;
      }
      Serial.print("PWM duty = ");
      Serial.println(PWM);
      break;
      
    case 'f': // ソフトウェアのバージョン情報表示
      Serial.println("******************************");
      Serial.println("xv11_13.ino");
      Serial.println("Creation Date: 2016/03/21");
      Serial.println("Copyright(c) 2016 Ito Taichi");
      Serial.println("All Right Reserved");
      Serial.println("******************************");
      break;
      
    default: // 無効なコマンドの場合
      Serial.println("Your command was ignored"); // 貴様のコマンドは無視されました
      break;
  }
  
  return 1;
  
}

