// XV11を接続したArduinoと連携する

import controlP5.*; //GUI LIB
import processing.serial.*;//Serial LIB

ControlP5 controlP5;//GUIオブジェクト
Serial myPort;//シリアルオブジェクト

String Cmd = ""; //Arduinoに送るコマンド
String Cmd_old = ""; //Arduinoに送ったコマンド

int valid_flag,angle,distance,speed,pwm=75; // Arduinoより送られるXV11のデータ

int myColorBackground=color(0,0,0); //バックグラウンドカラーを黒とする
int i;
int bx,by;//ボタンサイズ
int bwx,bwy;//ボタンコントローラの表示開始位置
int wsize_x=375; //windowのサイズ iPhone6/ iPhone6sと同サイズ
int wsize_y=667;

int magnification=1; // 拡大倍率


void setup(){
  size(wsize_x,wsize_y); //windowのサイズ
  frameRate(30); //表示切替時間間隔
  controlP5 = new ControlP5(this); //クラスのコピー
  myPort = new Serial(this,"COM14",115200); //使うCOMポート 必ずチェック&大文字で書くこと
  
  bwx=50; // ボタンコントローラの表示開始位置
  bwy=500;// ボタンコントローラの表示開始位置
  bx=by=35;//ボタンサイズ
  
  //control interface GUI
  //GUI部品にidを設定
  controlP5.addSlider("SPEED",0,255,75,50+bwx,50+bwy,200,20).setId(5);
  //slider function ("name",start V,end V,set start V,x,y,w,h,slide H)
  //function slider <- Slider val
  i=0; // start
  controlP5.addBang("Start",bwx,bwy,bx,by).setId(i);
  i=1; // end
  controlP5.addBang("End",bwx,50+bwy,bx,by).setId(i);
  i=2; // 画面クリア
  controlP5.addBang("Clear",bwx,120+bwy,bx,by).setId(i);
  i=3; // 拡大
  controlP5.addBang("Zoom In",50+bwx,120+bwy,bx,by).setId(i);
  i=4; // 縮小
  controlP5.addBang("Zoom Out",100+bwx,120+bwy,bx,by).setId(i);
  
  clear_screen(); // 画面クリア
  

  
}
  
void draw(){
  String dmy; //画面表示用文字列作成文字列
  
  // ボタンエリアの背景を黒にする。これは毎回行わないと文字の部分に過去に画面に描いたものが残る
  fill(80,80,80); // 色の指定 灰色で塗りつぶす
  noStroke(); // 太さの指定 太さなし
  rect(bwx-10,bwy-10,300,120);
  fill(64,64,64);
  rect(bwx-10,bwy+110,300,60);
 
  //シリアルに毎回データを送りまくるのはエレガントではないので値の更新があったときのみ送信
  if(Cmd.equals(Cmd_old)){
  }
  else{
    println(Cmd);
    myPort.write(Cmd);
  }
  Cmd_old=Cmd;
  
  // 画面に文字を表示する
  PFont font= createFont("MS Gothic",14);
  textFont(font);
  fill(255,255,255);
  strokeWeight(1);
  
  text("Speed "+speed+" [rpm]",bwx+50,bwy+12);
  text("PWM duty "+pwm,bwx+50,bwy+32);
  text("x 1/"+magnification,bwx+200,bwy+150);
  
  plot(magnification); // 指定された拡大倍率で距離をプロットする
  
}


void controlEvent(ControlEvent theEvent){
  int dmy;
  println(
  "## controlEvent / id:"+theEvent.controller().id()+
  " / name:"+theEvent.controller().name()+
  " / label:"+theEvent.controller().label()+
  " / value:"+theEvent.controller().value()
  );
  if(theEvent.controller().id()==0){ // スタート
    println("Start\n");
    Cmd = "s\r";
  }
  if(theEvent.controller().id()==1){ // ストップ
    println("End\n");
    Cmd = "e\r";
  }
  if(theEvent.controller().id()==5){ // duty比設定
    dmy = (int)theEvent.controller().value();
    Cmd = String.format("p,%d\r",dmy);
    println(Cmd);
  }
  if(theEvent.controller().id()==2){ // 画面クリア
    clear_screen(); // 画面クリア
  }
  if(theEvent.controller().id()==3){ // ズームイン
    clear_screen(); // 画面クリア
    magnification = magnification/2;
    if(magnification<1){ //1以下にはさせない
      magnification=1;
    }
  }
  if(theEvent.controller().id()==4){ // ズームアウト
    clear_screen(); // 画面クリア
    magnification= magnification*2;
    if(magnification>128){ //128以上にはさせない
      magnification=128;
    }
  }
}


void serialEvent(Serial p){
  int count = 7;
  // データフォーマット ,val,val,val,val,val,val\n
  
  //文字列の変数stringDataを用意
  //改行'\n'が来るまで読み込む
  String stringData=myPort.readStringUntil('\n');
 //   println(stringData);
  //文字列データが空でなければ
  if(stringData!=null){
    //文字列データに含まれる改行記号を取り除く
    stringData=trim(stringData);
     println(stringData);
    //整数型の配列data[]を用意し、
    //コンマ記号を元に文字列データを区切って
    //配列data[]に整数化して入れておく
    int data[]=int(split(stringData,','));
    
    //文字列データの先頭に,が付いているため、データ数は7となる
    //配列data[]内のデータが7つなら
    if(data.length==count){
      // 2番目のデータは有効フラグ
      valid_flag=data[2];
      // 3番目のデータは角度
      if(valid_flag==1){ // 有効フラグが1なら
        angle=data[3];
      }
      // 4番目のデータは距離
      if(valid_flag==1){
        distance=data[4];
      }
      // 5番目のデータは回転速度
      if(valid_flag==1){
        speed=data[5];
      }
      // 6番目のデータはduty比
      pwm=data[6];
    }
  }
}


// 指定された拡大倍率でプロットする関数
void plot(int magnification){ 
  float x=(distance * cos((angle+90) * PI/180)) / magnification + wsize_x/2;
  float y=(-distance * sin((angle+90) * PI/180)) / magnification + bwy/2; // y座標は下に向かい大きくなるので-をつける
  stroke(255,192,0); // 色の指定 黄色
  strokeWeight(3); // 太さの指定
  if(distance!=0) { // 原点座標にはプロットしない
    point(x,y);
  }
}

// 画面をリセットする関数
void clear_screen(){
  background(myColorBackground);// 背景を黒にする
  stroke(255,255,255); // 色の指定 白
  strokeWeight(1); // 太さの指定
//  point(wsize_x/2,bwy/2);
  // 原点に十字を描く
  line(wsize_x/2,bwy/2-5,wsize_x/2,bwy/2+5); // 縦線
  line(wsize_x/2-5,bwy/2,wsize_x/2+5,bwy/2); // 横線
  distance=0; // その後へんなところにプロットされないように距離をリセット
}
      
